import time
import json
import os
from google.appengine.ext import webapp
from google.appengine.ext.webapp.util import run_wsgi_app
from google.appengine.ext import db
from google.appengine.ext.webapp import template


class State(db.Model):
  
  content = db.TextProperty()
  
  @classmethod
  def singleton(self):
    return State.get_or_insert('default', content='{}')


def update_content(key, delta):
  state = db.get(key)
  content = json.read(state.content.encode('utf-8'))
  for k, v in delta.iteritems():
    if v is None:
      del content[k]
    else:
      content[k] = v
  state.content = json.write(content)
  state.put()


class IndexPage(webapp.RequestHandler):
  
  def get(self):
    self.response.headers['Content-Type'] = 'text/html; charset=utf-8'
    path = os.path.join(os.path.dirname(__file__), 'comet_test.html')
    self.response.out.write(template.render(path, {}))


class GetPage(webapp.RequestHandler):
  
  def post(self):
    
    last_content = self.request.get('last')
    for i in xrange(20):
      content = State.singleton().content
      if content != last_content:
        break
      time.sleep(1)
    
    self.response.headers['Content-Type'] = 'text/plain'
    self.response.out.write(content)


class SubmitPage(webapp.RequestHandler):
  
  def post(self):
    
    delta = json.read(self.request.get('delta').encode('utf-8'))
    state = State.singleton()
    db.run_in_transaction(update_content, state.key(), delta)
    
    self.response.headers['Content-Type'] = 'text/plain'
    self.response.out.write('Done')


application = webapp.WSGIApplication(
    [
      ('/comet-test', IndexPage),
      ('/comet-test/get', GetPage),
      ('/comet-test/submit', SubmitPage),
    ],
    debug=True)

def main():
  run_wsgi_app(application)

if __name__ == "__main__":
  main()
